﻿local Engine, Api, Locale, Settings = unpack(Vendetta)

-- Extensions
local debug = Engine.Core.Debug
local length = Engine.Core.Length

-- Registration
local Default = Engine:RegisterModule("Default")
LibStub("LibNameplateRegistry-1.0"):Embed(Default)

-- Initialize the module.
function Default:OnInitialize()
	UnitPopupButtons["VENDETTA"] = { text = Locale.DEFAULT_DROPDOWN_VENDETTA, dist = 0, nested = 1 }
	UnitPopupButtons["VENDETTA_MARK1"] = { text = Locale.DEFAULT_DROPDOWN_VENDETTA_MARK1, dist = 0 }
	UnitPopupButtons["VENDETTA_MARK2"] = { text = Locale.DEFAULT_DROPDOWN_VENDETTA_MARK2, dist = 0 }
	UnitPopupButtons["VENDETTA_MARK3"] = { text = Locale.DEFAULT_DROPDOWN_VENDETTA_MARK3, dist = 0 }
	UnitPopupButtons["VENDETTA_UNMARK"] = { text = Locale.DEFAULT_DROPDOWN_VENDETTA_UNMARK, dist = 0 }

	table.insert(UnitPopupMenus["PLAYER"], length(UnitPopupMenus["PLAYER"]), "VENDETTA")
	UnitPopupMenus["VENDETTA"] = { "VENDETTA_MARK1", "VENDETTA_MARK2", "VENDETTA_MARK3", "VENDETTA_UNMARK" }

	hooksecurefunc("UnitPopup_OnClick", function(self)
		local dropdownFrame = UIDROPDOWNMENU_INIT_MENU
		local unit = dropdownFrame.unit
		local button = self.value

		if button == "VENDETTA_MARK1" or button == "VENDETTA_MARK2" or button == "VENDETTA_MARK3" then
			local rating = 1
			if button == "VENDETTA_MARK2" then
				rating = 2
			elseif button == "VENDETTA_MARK3" then
				rating = 3
			end

			Default:Mark(unit, rating)
		elseif button == "VENDETTA_UNMARK" then
			Default:Unmark(unit)
		end
	end)
end

-- Called when the module is enabled.
function Default:OnEnable()
	self:LNR_RegisterCallback("LNR_ON_NEW_PLATE") -- registering this event will enable the library else it'll remain idle
    self:LNR_RegisterCallback("LNR_ON_RECYCLE_PLATE")
    self:LNR_RegisterCallback("LNR_ON_GUID_FOUND")
    self:LNR_RegisterCallback("LNR_ERROR_FATAL_INCOMPATIBILITY")
end

-- Called when the module is disabled.
function Default:OnDisable()
	self:LNR_UnregisterAllCallbacks()
end

-- Register the default settings for the module.
function Default:OnRegisterDefaults()
	return {
		Enabled = true
	}
end

function Default:LNR_ON_NEW_PLATE(event, frame, data)
    debug(ADDON_NAME, data.name, "'s nameplate appeared! It's a", data.type, "and", data.reaction, data.GUID and ("we know its GUID: " .. data.GUID) or "GUID not yet known")

	if Api.Elements[data.GUID] then
		local element = Api.Elements[data.GUID]
		local vendetta = self:GetVendettaFrame(frame)
		
		vendetta:Mark(element.Rating)
		element:Encounter()
	end
end


function Default:LNR_ON_RECYCLE_PLATE(event, frame, data)
	debug(ADDON_NAME, data.name .. "'s nameplate has disappeared!")

	if frame.Vendetta then
		frame.Vendetta:Unmark()
	end
end


function Default:LNR_ON_GUID_FOUND(event, frame, guid, method)
    -- This is now rarely useful since WoW 7 since GUIDs are linked directly on nameplate appearance.
    -- Sometimes though some data about a unit may not be available right away due to heavy lag.
    debug(ADDON_NAME, ":", "GUID found using", findmethod, "for", self:GetPlateName(frame), "'s nameplate:", guid)
end


function Default:LNR_ERROR_FATAL_INCOMPATIBILITY(event, icompatibilityType)
    -- Here you want to check if your add-on and LibNameplateRegistry are not
    -- outdated (old TOC) and display a nice error message to your user.
end

function Default:Mark(unit, rating)
	local guid = UnitGUID(unit)
	local class, _, race, _, gender, name, realm = GetPlayerInfoByGUID(guid)

	-- Mark the player as a vendetta target
	local element = Api:RegisterElement(guid, { Name = name, Realm = realm, Class = class, Race = race, Gender = gender, Rating = rating })

	-- Save element to database
	if element ~= nil then
		Settings.Database.global.Elements[element.Guid] = { Guid = element.Guid, Name = element.Name, Realm = element.Realm, Class = element.Class, Race = element.Race, Gender = element.Gender, Rating = element.Rating, Notes = element.Notes, Timestamp = element.Timestamp }
		
		-- Mark frames that are already in view
		local frame, data = self:GetPlateByGUID(guid)
		if frame ~= nil then
			local vendetta = self:GetVendettaFrame(frame)
			vendetta:Mark(element.Rating)
		end
	end
end

function Default:Unmark(unit, rating)
	local guid = UnitGUID(unit)

	-- Unregister element and remove from database
	Api:UnregisterElement(guid)
	Settings.Database.global.Elements[guid] = nil
	
	local frame, data = self:GetPlateByGUID(guid)
	if frame ~= nil then
		local vendetta = self:GetVendettaFrame(frame)
		vendetta:Unmark()
	end
end

function Default:GetVendettaFrame(frame)
	if not frame.Vendetta then
		local vendetta = CreateFrame("Frame", "VendettaMarkerFrame", frame)
	
		vendetta:SetPoint("RIGHT", frame, "LEFT")
		vendetta:SetWidth(32)
		vendetta:SetHeight(32)

		vendetta.Icon = vendetta:CreateTexture(nil, "BACKGROUND")
		vendetta.Icon:SetTexture("Interface\\AddOns\\Vendetta\\Media\\Vendetta")
		vendetta.Icon:SetWidth(32)
		vendetta.Icon:SetHeight(32)
		vendetta.Icon:SetPoint("TOPLEFT")

		vendetta.Star1 = vendetta:CreateTexture(nil, "BACKGROUND")
		vendetta.Star1:SetTexture("Interface\\AddOns\\Vendetta\\Media\\Star")
		vendetta.Star1:SetWidth(10)
		vendetta.Star1:SetHeight(10)
		vendetta.Star1:SetPoint("BOTTOMLEFT")

		vendetta.Star2 = vendetta:CreateTexture(nil, "BACKGROUND")
		vendetta.Star2:SetTexture("Interface\\AddOns\\Vendetta\\Media\\Star")
		vendetta.Star2:SetWidth(10)
		vendetta.Star2:SetHeight(10)
		vendetta.Star2:SetPoint("LEFT")

		vendetta.Star3 = vendetta:CreateTexture(nil, "BACKGROUND")
		vendetta.Star3:SetTexture("Interface\\AddOns\\Vendetta\\Media\\Star")
		vendetta.Star3:SetWidth(10)
		vendetta.Star3:SetHeight(10)
		vendetta.Star3:SetPoint("TOPLEFT")

		vendetta.Mark = function(self, rating)
			vendetta:SetAlpha(1)
			vendetta.Icon:SetAlpha(1)
			
			vendetta.Star1:SetAlpha(0)
			vendetta.Star2:SetAlpha(0)
			vendetta.Star3:SetAlpha(0)

			if rating == 1 then
				vendetta.Star1:SetAlpha(1)
			elseif rating == 2 then
				vendetta.Star1:SetAlpha(1)
				vendetta.Star2:SetAlpha(1)
			elseif rating == 3 then
				vendetta.Star1:SetAlpha(1)
				vendetta.Star2:SetAlpha(1)
				vendetta.Star3:SetAlpha(1)
			end
		end

		vendetta.Unmark = function(self)
			frame.Vendetta:SetAlpha(0)
			frame.Vendetta.Icon:SetAlpha(0)
			frame.Vendetta.Star1:SetAlpha(0)
			frame.Vendetta.Star2:SetAlpha(0)
			frame.Vendetta.Star3:SetAlpha(0)
		end
	
		frame.Vendetta = vendetta
	end

	return frame.Vendetta
end